#!/bin/sh
#----------------------------------------------------------------------------
# ipv6-configure-tunnel.sh                                       3.10.4
#
#
# Last Update:  $Id: ipv6-configure-tunnel.sh 27199 2013-04-19 13:30:35Z kristov $
#----------------------------------------------------------------------------

# $1 = file containing IPv6 tunnel data
# $2 = prefixaddr/prefixlen (can be empty)
# $3 = valid lifetime of prefix (can be empty if $2 is)
# $4 = preferred lifetime of prefix (can be empty if $2 is)

# include IPv4 helper functions (especially log_*)
. /etc/boot.d/base-helper

net_config_dir="/var/run/ipv6.nets"

tunnelfile="$1"
if [ ! -f "$tunnelfile" ]
then
    log_error "$(basename $0): IPv6 tunnel information file '$tunnelfile' does not exist"
    exit 1
fi

tunnelname="IPV6_TUNNEL_$(basename "$tunnelfile" .conf)"
. "$tunnelfile"
oldprefix="$prefix"

if [ -n "$2" ]
then
    prefix=$(expandv6.sh "$2")
    if [ "$prefix" != "$(prefixcalc6 $prefix)" ]
    then
        log_error "$(basename $0): $tunnelname: wrong prefix $2 received"
        exit 2
    fi

    prefixaddr=$(echo $prefix | cut -d '/' -f 1)
    prefixlen=$(echo $prefix | cut -d '/' -f 2)
    prefixvalidlft=$3
    prefixpreflft=$4
else
    prefix=
    prefixaddr=
    prefixlen=
    prefixvalidlft=
    prefixpreflft=
fi

sed -i "s,^prefix=.*,prefix=$prefix," "$tunnelfile"

# default routes via PPP tunnels are handled by imond...
if [ -n "$oldprefix" ]
then
    if [ "$default" = "yes" ]
    then
        rm -f /var/run/ipv6.tunnel.current
        if [ "$type" != "ppp" ]
        then
            run-ipv6-updown.sh down $tunneldev $oldprefix
            log_info "$(basename $0): $tunnelname: removing default route from device $tunneldev"
            ip -6 route del default dev $tunneldev
        fi
    fi
    ip -6 route del unreachable $oldprefix
fi

for netfile in "$net_config_dir"/*.conf
do
    [ -f "$netfile" ] || continue
    . "$netfile"

    if [ "$tunnel" = "$index" ]
    then
        netname="IPV6_NET_$(basename "$netfile" .conf)"
        if [ -n "$ifnet" ]
        then
            ipv6-interface-down.sh "$netfile"
        fi
        if [ -n "$prefix" ]
        then
            ifaddr=$(addip6 $prefixaddr $netaddr)
            if [ $prefixlen -gt $netlen -o "$prefix" != "$(prefixcalc6 $ifaddr/$prefixlen)" ]
            then
                log_error "$(basename $0): $netname: prefix overflow: received prefix $prefix is too long to encompass subnet $netaddr/$netlen (results in $ifaddr/$netlen)"
                continue
            fi

            ipv6-interface-up.sh "$netfile" $ifaddr/$netlen "$prefixvalidlft" "$prefixpreflft"
        fi
    fi
done

# default routes via PPP tunnels are handled by imond...
if [ -n "$prefix" ]
then
    # see http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=562077 for an explanation why this unreachable routes is necessary
    ip -6 route add unreachable $prefix
    if [ "$default" = "yes" ]
    then
        if [ "$type" != "ppp" ]
        then
            log_info "$(basename $0): $tunnelname: adding default route via device $tunneldev"
            ip -6 route add default dev $tunneldev
            run-ipv6-updown.sh up $tunneldev $prefix $prefixvalidlft $prefixpreflft
        fi
        ln -sf "$tunnelfile" /var/run/ipv6.tunnel.current
    fi
fi
