#!/bin/sh
#----------------------------------------------------------------------------
# ipv6-interface-down.sh                                         3.10.4
#
#
# Last Update:  $Id: ipv6-interface-down.sh 28088 2013-07-18 13:33:14Z kristov $
#----------------------------------------------------------------------------

# $1 = file containing IPv6 network data

# include IPv4 helper functions (especially log_* and translate_ip_net)
. /etc/boot.d/base-helper
# include IPv6 helper functions (especially fliplocalbit)
. /etc/rc.d/base-helper.ipv6

dhcpdevices=/var/run/dhcpd6.if

netfile="$1"
if [ ! -f "$netfile" ]
then
    log_error "$(basename $0): IPv6 network information file '$netfile' does not exist"
    exit 1
fi

. "$netfile"

if [ ! -n "$ifnet" ]
then
    # nothing to do
    exit 0
fi

netname="IPV6_NET_$(basename "$1" .conf)"
net=$ifnet
netaddr=$(echo $net | cut -d '/' -f 1)
netlen=$(echo $net | cut -d '/' -f 2)

prefix=$(prefixcalc6 $net)
prefixaddr=$(echo $prefix | cut -d '/' -f 1)

sed -i "s,^ifnet=.*,ifnet=," "$netfile"

eval $(grep "^\(HOSTNAME\|DOMAIN_NAME\|OPT_DNS\|DNS_SUPPORT_IPV6\|DNS_LISTEN_N\)=" /etc/rc.cfg)

if [ "$adv" = "yes" ]
then
    rm -f /var/run/radvd-$dev.conf
    > /var/run/reload.radvd
    log_info "$(basename $0): $netname: stopped advertising prefix $prefix on $dev"
fi

if [ "$dhcp" = "yes" ]
then
    sed -i "/^$dev$/d" "$dhcpdevices"
    rm -f /var/run/dhcpd6-$dev.conf
    > /var/run/reload.dhcpd6
fi

if [ "$OPT_DNS" = "yes" -a "$DNS_SUPPORT_IPV6" = "yes" -a "$DNS_LISTEN_N" != "0" ]
then
    if [ "$dhcp" = "yes" -o "$adv_dns" = "yes" -a -f /etc/dnsmasq.d/dns_ipv6.conf ]
    then
        sed -i "/^listen-address=$netaddr$/d" /etc/dnsmasq.d/dns_ipv6.conf
        > /var/run/reload.dnsmasq
    fi
fi

ip -6 addr del $net dev $dev
log_info "$(basename $0): $netname: address $netaddr removed from $dev"

dnsdev=$(echo -n $dev | sed 's/\./-vlan-/;s/:/-alias-/')
sed -i "/^$netaddr $canonical $HOSTNAME-$dnsdev.$DOMAIN_NAME $HOSTNAME-$dnsdev$/d" /etc/hosts

if [ "$OPT_DNS" = "yes" -a "$DNS_SUPPORT_IPV6" = "yes" -a $netlen -eq 64 ]
then
    eval $(grep "^HOST_\(N\|[0-9]\+_\(IP[46]\|IP6_NET\|MAC\)\)=" /etc/rc.cfg)
    for idx in $(seq 1 $HOST_N)
    do
        eval ip6='$HOST_'$idx'_IP6'
        eval ip6net='$HOST_'$idx'_IP6_NET'
        eval mac='$HOST_'$idx'_MAC'
        : ${ip6net:="IPV6_NET_1"}

        if [ "$ip6" = "auto" -a "$ip6net" = "$netname" -a -n "$mac" ]
        then
            eval ip4='$HOST_'$idx'_IP4'
            if translate_ip_net $ip4 HOST_${idx}_IP4
            then
                ip4=$res
            fi

            # compute EUI-64 from MAC
            set -- $(echo -n $mac | cut -d ':' -f 1 | sed 's/^\(.\)\(.\).*$/\1 \2/')
            first=$1
            fliplocalbit $2

            eui="$first$result$(echo -n $mac | cut -d ':' -f 2,3)ff:fe$(echo -n $mac | cut -d ':' -f 4,5)$(echo -n $mac | cut -d ':' -f 6)"
            addr=$(echo -n $prefixaddr | sed "s/0:0:0:0$/$eui/")
            for f in /etc/hosts /etc/hosts.d/hosts.*
            do
                [ -f "$f" ] || continue
                sed -i "/^$addr /d;s/^#\(#*::ffff:$ip4 .*\)/\1/" $f
            done
        fi
    done
fi

exit 0
