#!/bin/sh
#----------------------------------------------------------------------------
# /usr/local/bin/setup-dynamic-ip6tables.sh - setup dynamic IPv6 packet filter
#
# Last Update:  $Id: setup-dynamic-ip6tables.sh 38172 2015-04-03 05:45:09Z kristov $
#----------------------------------------------------------------------------

[ -f /var/run/ip6_net.conf ] || exit 0

. /etc/boot.d/base-helper
. /etc/rc.d/base-helper.ipv6
. /etc/rc.d/fwrules-helper
. /etc/rc.d/fwrules-helper.ipv6

handle_chain()
{
    local table=$1 chain=$2 method=$3 ok=$4

    local processed=
    local delta=0
    local added=0
    ip6tables -t $table -nL $chain --line-numbers | grep "$dummy_comment \*/" | while read pos rest
    do
        if [ -n "$processed" ]
        then
            ip6tables -t $table -D $chain $((pos+added-delta))
            delta=$((delta+1))
        elif [ $ok -eq 1 ]
        then
            do_rule6 "$table" "$chain" R "$rule" $pos $method "$comment"
            added=$((ip6tables_rules_added-1))
        else
            do_rule6 "$table" "$chain" R "$dummy_rule" $pos $method "$comment"
            added=$((ip6tables_rules_added-1))
        fi
        processed=1
    done
}

for i in $(seq 1 $(cat $ip6tables_dynrules_idx))
do
    rulefile=$ip6tables_dynrules/$i.rule
    [ -f "$rulefile" ] || continue
    . "$rulefile"

    ok=1
    for ip in $dyn_ips
    do
        if ! can_translate_ip6_net $ip
        then
            ok=0
            break
        fi
    done

    handle_chain $table $chain exec_rule6 $ok
    case $chain in
    PORTFW)
        handle_chain filter PORTFWACCESS exec_access_rule6 $ok
        handle_chain filter PORTREDIRACCESS exec_access_rule6 $ok
        ;;
    esac
done
